use strict;
## no critic warnings

package Email::MIME::Modifier;

use vars qw[$VERSION];
$VERSION = '1.910';

use Email::MIME;

1;

=head1 NAME

Email::MIME::Modifier - obsolete do-nothing library

=head1 VERSION

version 1.910

=head1 PERL EMAIL PROJECT

This module is maintained by the Perl Email Project

L<http://emailproject.perl.org/wiki/Email::MIME>

=head1 AUTHOR

Casey West, <F<casey@geeknest.com>>.

=head1 COPYRIGHT

  Copyright (c) 2004 Casey West.  All rights reserved.
  This module is free software; you can redistribute it and/or modify it
  under the same terms as Perl itself.

=cut
